@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 # -----------------------------------------------------------------------
 # Copyright (c) 2009  Jendrik Seipp
@@ -19,6 +18,7 @@
 # -----------------------------------------------------------------------
 
 import os
+import logging
 
 from rednotebook.util import filesystem
 
@@ -31,8 +31,9 @@ class Archiver(object):
 		archiveFiles = []
 		for root, dirs, files in os.walk(dataDir):
 			for file in files:
-				archiveFiles.append(os.path.join(root, file))
+				if not file.endswith('~'):
+					archiveFiles.append(os.path.join(root, file))
 		
 		filesystem.writeArchive(backup_file, archiveFiles, dataDir)
 		
-		print 'The content has been exported to %s' % backup_file
\ No newline at end of file
+		logging.info('The content has been exported to %s' % backup_file)
