@@ -26,8 +26,10 @@ import gtk
 import gobject
 
 
-from rednotebook.gui.widgets import UrlButton, CustomComboBoxEntry
-from rednotebook.util import filesystem
+from rednotebook.gui.customwidgets import UrlButton, CustomComboBoxEntry
+from rednotebook.gui.customwidgets import ActionButton
+from rednotebook.gui import browser
+from rednotebook.util import filesystem, utils
 from rednotebook import info
 
 class Option(gtk.HBox):
@@ -117,6 +119,9 @@ class CsvTextOption(Option):
 		# directly read the string, not the list
 		values_string = Option.config.read(option_name, '')
 		
+		# Ensure that we have a string here
+		values_string = str(values_string)
+		
 		self.entry = gtk.Entry()
 		self.entry.set_text(values_string)
 		
@@ -172,6 +177,7 @@ class DateFormatOption(ComboBoxOption):
 		
 	def on_format_changed(self, widget):
 		import time
+		### Translators: Noun
 		self.preview.set_text(_('Preview: %s') % time.strftime(self.combo.get_active_text()))
 		
 class FontSizeOption(ComboBoxOption):
@@ -277,26 +283,49 @@ class OptionsManager(object):
 		
 		if platform.system() == 'Linux' and os.path.exists('/usr/bin/rednotebook'):
 			logging.debug('Running on Linux. Is installed. Adding autostart option')
-			self.options.insert(0, AutostartOption())
-		
-		self.options.append(TickOption(_('Check for new version at startup'), 'checkForNewVersion'))
-		
+			self.options.insert(0, AutostartOption())		
 		
 		self.options.append(TickOption(_('Close to system tray'), 'closeToTray',
 				tooltip=_('Closing the window will send RedNotebook to the tray')))
 		
 		able_to_spell_check = self.main_window.dayTextField.can_spell_check()
 		tooltip = _('Underline misspelled words') if able_to_spell_check else \
-				_('Requires gtkspell. This is included in the python-gnome2-extras package')
-		spell_check_option = TickOption(_('Spell Check'), 'spellcheck',
+				_('Requires gtkspell.') + ' ' + \
+				_('This is included in the python-gtkspell or python-gnome2-extras package')
+		spell_check_option = TickOption(_('Check Spelling'), 'spellcheck',
 				tooltip=tooltip)
 		if not sys.platform == 'win32':
 			self.options.append(spell_check_option)
 		spell_check_option.set_sensitive(able_to_spell_check)
 		
+		webkit_available = bool(browser.webkit)
+		tooltip = _('Webkit offers a nicer preview') if webkit_available else \
+				_('Requires pywebkitgtk (python-webkit)')
+		webkit_label = _('Use webkit for previews') + ' ' + _('(Restart required)')
+		webkit_option = TickOption(webkit_label, 'useWebkit',
+				tooltip=tooltip)
+		if not sys.platform == 'win32':
+			self.options.append(webkit_option)
+		webkit_option.set_sensitive(webkit_available)
+		
+		
+		# Check for new version
+		
+		check_version_option = TickOption(_('Check for new version at startup'), 'checkForNewVersion')
+		def check_version_action(widget):
+			utils.check_new_version(self.main_window, info.version)
+			# Apply changes from dialog to options window
+			check = bool(self.redNotebook.config.get('checkForNewVersion'))
+			check_version_option.check_button.set_active(check)
+			
+		check_version_button = ActionButton(_('Check now'), check_version_action)
+		check_version_option.pack_start(check_version_button, False, False)
+		self.options.append(check_version_option)
+		
+		
 		self.options.extend([
-				DateFormatOption(_('Date/Time format'), 'dateTimeString'),
 				FontSizeOption(_('Font Size'), 'mainFontSize'),
+				DateFormatOption(_('Date/Time format'), 'dateTimeString'),
 				CsvTextOption(_('Exclude from clouds'), 'cloudIgnoreList', \
 								tooltip=_('Do not show those comma separated words in any cloud')),
 				CsvTextOption(_('Allow small words in clouds'), 'cloudIncludeList', \
