@@ -28,18 +28,18 @@ import time
 from optparse import OptionParser, OptionValueError
 
 
-	
-
-
 if hasattr(sys, "frozen"):
 	from rednotebook.util import filesystem
 	from rednotebook.util import utils
 	from rednotebook import info
+	from rednotebook import configuration
 else:
 	from util import filesystem # creates a copy of filesystem module
 	#import util.filesystem # imports the original filesystem module
 	from util import utils
 	import info
+	import configuration
+	
 	
 
 def parse_options():
@@ -48,12 +48,7 @@ def parse_options():
 						  #option_class=ExtOption,
 						  formatter=utils.IndentedHelpFormatterWithNL(),
 						  )
-#	parser.add_option(
-#		'-p', '--portable', dest='portable', default=False,
-#		action='store_true', 
-#		help='Run in portable mode ' \
-#			'(default: False)')
-	
+						
 	parser.add_option(
 		'-d', '--debug', dest='debug', \
 		default=False, action='store_true',
@@ -122,7 +117,10 @@ def setup_logging(log_file):
 	logging.info('Writing log to file "%s"' % log_file)
 
 
-dirs = filesystem.Filenames()
+default_config_file = os.path.join(filesystem.appDir, 'files', 'default.cfg')
+default_config = configuration.Config(default_config_file)
+
+dirs = filesystem.Filenames(default_config)
 setup_logging(dirs.logFile)
 
 ## ------------------ end Enable logging -------------------------------
@@ -134,14 +132,26 @@ setup_logging(dirs.logFile)
 # set the locale for all categories to the user’s default setting 
 # (typically specified in the LANG environment variable)
 import locale
+lang = os.environ.get('LANG', None)
+logging.debug('LANG: %s' % lang)
+default_locale = locale.getdefaultlocale()[0]
+logging.debug('Default locale: %s' % default_locale)
 try:
 	locale.setlocale(locale.LC_ALL, '')
+	logging.debug('Set default locale: "%s"' % default_locale)
 except locale.Error, err:
 	# unsupported locale setting
-	lang = os.getenv('LANG')
-	logging.error('Locale "%s" could not be set: "%s"' % (lang, err))
+	logging.error('Locale "%s" could not be set: "%s"' % (default_locale, err))
 	logging.error('Probably you have to install the appropriate language packs')
 
+# If the default locale could be determined and the LANG env variable
+# has not been set externally, set LANG to the default locale
+# This is necessary only for windows where program strings are not
+# shown in the system language, but in English
+if default_locale and not lang:
+	logging.debug('Setting LANG to %s' % default_locale)
+	os.environ['LANG'] = default_locale
+	
 LOCALE_PATH = os.path.join(dirs.appDir, 'i18n')
 
 # the name of the gettext domain. because we have our translation files
@@ -152,28 +162,31 @@ GETTEXT_DOMAIN = 'rednotebook'
 # set up the gettext system
 import gettext
 
-modules = [gettext]
+# Adding locale to the list of modules translates gtkbuilder strings
+modules = [gettext, locale]
 
-# Little Hack: The C function bindtextdomain() has to be called for
-# gtkbuilder stuff to be translated. This is not yet done by gtkbuilder.
-# As a workaround we use gtk.glade.bindtextdomain
+# Sometimes this doesn't work though, 
+# so we try to call gtk.glade's function as well if glade is present
 try:
 	import gtk.glade
 	modules.append(gtk.glade)
 except ImportError, err:
-	logging.warning('Importing gtk.glade failed. Some strings may not be translated')
+	pass
 
 for module in modules:
-    module.bindtextdomain(GETTEXT_DOMAIN, LOCALE_PATH)
-    module.textdomain(GETTEXT_DOMAIN)
-    
-
-
-
+	try:
+		# locale.bintextdomain and locale textdomain not available on win
+		module.bindtextdomain(GETTEXT_DOMAIN, LOCALE_PATH)
+		module.textdomain(GETTEXT_DOMAIN)
+	except AttributeError, err:
+		logging.info(err)
+		
 # register the gettext function for the whole interpreter as "_"
 import __builtin__
 __builtin__._ = gettext.gettext
 
+
+
 ## ------------------- end Enable i18n -------------------------------
 
 
@@ -181,31 +194,31 @@ __builtin__._ = gettext.gettext
 
 try:
 	import pygtk
+	if not sys.platform == 'win32':
+		pygtk.require("2.0")
 except ImportError:
 	logging.error('pygtk not found. Please install PyGTK (python-gtk2)')
 	sys.exit(1)
 
-pygtk.require("2.0")
-
 try:
 	import gtk
-	#logging.disable(logging.DEBUG)
-	#logging.disable(logging.INFO)
-	#logging.disable(logging.ERROR)
-	#logging.disable(logging.WARNING)
-	#logging.disable(logging.CRITICAL)
-	gtk.gdk.threads_init()
-	#gtk.gdk.threads_enter()
 	
 	import gobject
-except (ImportError, AssertionError):
+	# Some notes on threads_init:
+	# only gtk.gdk.threads_init(): pdf export works, but gui hangs afterwards
+	# only gobject.threads_init(): pdf export works, gui works
+	# both: pdf export works, gui hangs afterwards	
+	gobject.threads_init() # only initializes threading in the glib/gobject module
+	#gtk.gdk.threads_init() # also initializes the gdk threads
+except (ImportError, AssertionError), e:
+	logging.error(e)
 	logging.error('gtk not found. Please install PyGTK (python-gtk2)')
 	sys.exit(1)
 
 try:
 	import yaml
 except ImportError:
-	logging.error('Yaml is not installed (install python-yaml)')
+	logging.error('PyYAML not found. Please install python-yaml or PyYAML')
 	sys.exit(1)
 
 # The presence of the yaml module has been checked
@@ -223,21 +236,20 @@ logging.info('AppDir: %s' % filesystem.appDir)
 baseDir = os.path.abspath(os.path.join(filesystem.appDir, '../'))
 logging.info('BaseDir: %s' % baseDir)
 if baseDir not in sys.path:
-	logging.info('Adding BaseDir to sys.path')
+	# Adding BaseDir to sys.path
 	sys.path.insert(0, baseDir)
 	
 
-
-
 # This version of import is needed for win32 to work
 from rednotebook.util import unicode
 from rednotebook.util import dates
 #from rednotebook import info
-from rednotebook import config
+#from rednotebook import configuration
 from rednotebook import backup
 
-from rednotebook.gui.mainWindow import MainWindow
+
 from rednotebook.util.statistics import Statistics
+from rednotebook.gui.mainWindow import MainWindow
 
 
 
@@ -246,8 +258,15 @@ class RedNotebook:
 	def __init__(self):
 		self.dirs = dirs
 		
-		self.config = config.Config(self.dirs)
-		logging.info('Running in portable mode: %s' % self.config.read('portable', 0))
+		user_config = configuration.Config(self.dirs.configFile)
+		# Apply defaults where no custom values have been set
+		for key, value in default_config.items():
+			if key not in user_config:
+				user_config[key] = value
+		self.config = user_config
+		self.config.save_state()
+		
+		logging.info('Running in portable mode: %s' % self.dirs.portable)
 		
 		self.testing = False
 		if options.debug:
@@ -267,22 +286,19 @@ class RedNotebook:
 		# The dir name is the title
 		self.title = ''
 		
-		# show instructions at first start or if testing
-		default_dir_selected = (self.dirs.dataDir == self.dirs.defaultDataDir)
-		self.firstTimeExecution = default_dir_selected and \
-									len(os.listdir(self.dirs.dataDir)) == 0
-		logging.info('First Start: %s' % self.firstTimeExecution)
+		# show instructions at first start
+		logging.info('First Start: %s' % self.dirs.is_first_start)
 		
 		logging.info('RedNotebook version: %s' % info.version)
 		logging.info(filesystem.get_platform_info())
 		
 		utils.set_environment_variables(self.config)
 		
+		self.actualDate = datetime.date.today()
+		
 		# Let components check if the MainWindow has been created
 		self.frame = None
 		self.frame = MainWindow(self)
-		   
-		self.actualDate = datetime.date.today()
 		
 		self.open_journal(self.get_journal_path())
 		
@@ -303,7 +319,11 @@ class RedNotebook:
 		not present
 		'''
 		if not args:
-			return self.config.read('dataDir', self.dirs.dataDir)
+			data_dir = self.config.read('dataDir', self.dirs.dataDir)
+			if not os.path.isabs(data_dir):
+				data_dir = os.path.join(self.dirs.appDir, data_dir)
+				data_dir = os.path.normpath(data_dir)
+			return data_dir
 		
 		# path_arg can be e.g. data (under .rednotebook), data (elsewhere), 
 		# or an absolute path /home/username/myjournal
@@ -414,7 +434,7 @@ class RedNotebook:
 						month.edited = False
 					except OSError, err:
 						self.frame.show_save_error_dialog(exitImminent)
-						return True	
+						return True
 					except IOError, err:
 						self.frame.show_save_error_dialog(exitImminent)
 						return True	
@@ -422,8 +442,10 @@ class RedNotebook:
 		
 		if something_saved:
 			self.showMessage(_('The content has been saved to %s') % self.dirs.dataDir, error=False)
+			logging.info('The content has been saved to %s' % self.dirs.dataDir)
 		else:
 			self.showMessage(_('Nothing to save'), error=False)
+			logging.info('Nothing to save')
 		
 		if self.config.changed():
 			try:
@@ -431,6 +453,7 @@ class RedNotebook:
 				self.config.saveToDisk()
 			except IOError, err:
 				self.showMessage(_('Configuration could not be saved. Please check your permissions'))
+				logging.info('Configuration could not be saved. Please check your permissions')
 		
 		if not (exitImminent or changing_journal) and something_saved:
 			# Update cloud
@@ -484,7 +507,7 @@ class RedNotebook:
 		sortedCategories = sorted(self.nodeNames, key=lambda category: str(category).lower())
 		self.frame.categoriesTreeView.categories = sortedCategories
 		
-		if self.firstTimeExecution:
+		if self.dirs.is_first_start and data_dir_empty:
 			logging.info('Adding example content')
 			self.addInstructionContent()
 			
@@ -509,7 +532,11 @@ class RedNotebook:
 		self.frame.mainFrame.set_title(frame_title)
 		
 		# Save the folder for next start
-		self.config['dataDir'] = data_dir
+		if not self.dirs.portable:
+			self.config['dataDir'] = data_dir
+		else:
+			rel_data_dir = filesystem.get_relative_path(self.dirs.appDir, data_dir)
+			self.config['dataDir'] = rel_data_dir
 		
 		# Set the date range for the export assistant
 		start_date = self.getEditDateOfEntryNumber(0)
@@ -522,10 +549,11 @@ class RedNotebook:
 		
 	def loadAllMonthsFromDisk(self):
 		logging.debug('Starting to load files in dir "%s"' % self.dirs.dataDir)
-		for root, dirs, files in os.walk(self.dirs.dataDir):
-			for file in files:
-				if not file.endswith('~'):
-					self.loadMonthFromDisk(os.path.join(root, file))
+		#for root, dirs, files in os.walk(self.dirs.dataDir):
+		files = os.listdir(self.dirs.dataDir)
+		for file in files:
+			if not file.endswith('~'):
+				self.loadMonthFromDisk(os.path.join(self.dirs.dataDir, file))
 		logging.debug('Finished loading files in dir "%s"' % self.dirs.dataDir)
 	
 	
@@ -565,8 +593,9 @@ Filenames have to have the following form: 2009-01.txt \
 		except IOError:
 			#If that fails, there is nothing to load, so just display an error message
 			logging.error('Error: The file %s could not be read' % monthFileString)
-		except Exception:
-			logging.error('An error occured while reading %s' % monthFileString)
+		except Exception, err:
+			logging.error('An error occured while reading %s:' % monthFileString)
+			logging.error('%s' % err)
 		
 		
 	def loadMonth(self, date):
@@ -627,7 +656,6 @@ Filenames have to have the following form: 2009-01.txt \
 			
 	def showMessage(self, messageText, error=False, countdown=True):
 		self.frame.statusbar.showText(messageText, error, countdown)
-		logging.info(messageText)
 		
 		
 	def _getNodeNames(self):
