@@ -119,7 +119,7 @@ def _get_config(type):
 		config['postproc'].append(['(?i)(</?)s>', '\\1strike>'])
 		
 		# Allow line breaks, r'\\\\' are 2 \ for regexes
-		config['postproc'].append([r'\\\\', '<BR>'])
+		config['postproc'].append([r'\\\\', '<br />'])
 		
 	elif type == 'tex':
 		config['encoding'] = 'utf8'
@@ -131,6 +131,9 @@ def _get_config(type):
 		config['preproc'].append([r'\[""', r'["""'])
 		config['preproc'].append([r'""\.', r'""".'])
 		
+		# For images we have to omit the file:// prefix
+		config['postproc'].append([r'includegraphics\{"file://', r'includegraphics{"'])
+		
 		# Allow line breaks, r'\\\\' are 2 \ for regexes
 		config['postproc'].append([r'\$\\backslash\$\$\\backslash\$', r'\\\\'])
 		
